@@ -18,19 +18,26 @@ libcloud provides a unified interface to the cloud computing resources.
 
 :var __version__: Current version of libcloud
 """
-
-__all__ = ['__version__', 'enable_debug']
-__version__ = '1.0.0'
-
 import os
 import codecs
 
+from libcloud.base import DriverType  # NOQA
+from libcloud.base import DriverTypeFactoryMap  # NOQA
+from libcloud.base import get_driver  # NOQA
+
+
 try:
     import paramiko
     have_paramiko = True
 except ImportError:
     have_paramiko = False
 
+__all__ = [
+    '__version__',
+    'enable_debug'
+]
+__version__ = '2.1.0'
+
 
 def enable_debug(fo):
     """
@@ -39,13 +46,11 @@ def enable_debug(fo):
     :param fo: Where to append debugging information
     :type fo: File like object, only write operations are used.
     """
-    from libcloud.common.base import (Connection,
-                                      LoggingHTTPConnection,
-                                      LoggingHTTPSConnection)
-    LoggingHTTPSConnection.log = fo
-    LoggingHTTPConnection.log = fo
-    Connection.conn_classes = (LoggingHTTPConnection,
-                               LoggingHTTPSConnection)
+    from libcloud.common.base import Connection
+    from libcloud.utils.loggingconnection import LoggingConnection
+
+    LoggingConnection.log = fo
+    Connection.conn_class = LoggingConnection
 
 
 def _init_once():
